@@ -5,7 +5,7 @@ from typing import List, Union, Callable
 from collections import defaultdict
 
 from mcqa_utils.answer import Answer
-from mcqa_utils.utils import label_to_id
+from mcqa_utils.utils import label_to_id, id_to_label
 from mc_transformers.utils_mc import processors, DataProcessor, InputExample
 
 
@@ -113,22 +113,30 @@ class Dataset(object):
     def decode_id(self, id: str) -> str:
         return self.processor._decode_id(id)
 
-    def get_gold_answers(self, splits: Union[List[str], str]) -> List[Answer]:
+    def get_gold_answers(
+        self,
+        splits: Union[List[str], str],
+        with_text_values: bool = False
+    ) -> List[Answer]:
         answers = []
         data = self.get_splits(splits)
         for example in data:
             example_id = '-'.join(self.decode_id(example.example_id))
-            answers.append(Answer(
+            answer_dict = dict(
                 example_id=example_id,
                 label=example.label,
-                pred_label=example.label,
-            ))
+                pred_label=example.label
+            )
+            if with_text_values:
+                answer_dict.update(endings=example.endings)
+
+            answers.append(Answer(**answer_dict))
 
         return answers
 
-    def find_mask(self, split: Union[List[str], str], test_fn: Callable):
+    def find_mask(self, examples: List[InputExample], test_fn: Callable):
         mask = []
-        for sample in self.get_splits(split):
+        for sample in examples:
             if test_fn(sample):
                 mask.append(1)
             else:
@@ -149,7 +157,7 @@ class Dataset(object):
             json_ex = {
                 'id': id,
                 'article': grouped[0].contexts[0],
-                'answers': [chr(int(ex.label) + ord('A')) for ex in grouped],
+                'answers': [id_to_label(ex.label) for ex in grouped],
                 'options': [ex.endings for ex in grouped],
                 'questions': [ex.question for ex in grouped],
             }
@@ -185,6 +193,21 @@ class Dataset(object):
                 label = ord(ex.label.upper()) - ord('A')
             yield ex.example_id, ex.contexts[0], ex.question, ex.endings, label
 
+    def reduce_by_mask(
+        self, data: List, mask: Union[List[bool], Callable]
+    ) -> List:
+        if isinstance(mask, Callable):
+            mask = self.find_mask(data, mask)
+        return self._reduce_by_mask_list(data, mask)
+
+    def _reduce_by_mask_list(self, data: List, mask: List[bool]):
+        end_list = []
+        for point, keep in zip(data, mask):
+            if bool(keep):
+                end_list.append(point)
+
+        return end_list
+
     # deprecated
     def apply_no_answer(
         self,
